%{
	(* This file is the Ocamlyacc definition of the parser for the Kaleidoscope language. *)
	
	(* Standard libraries *)
	open Printf
	
	(* LLVM related libraries *)
	
	(* This project's libraries *)
	open KaleidoscopeAST
%}

%token <string> IDENT
%token <float> NUMBER
%token DEF EXTERN
%token SEMI LPAREN RPAREN
%token ASSIGN
%token PLUS SUB MUL DIV
%token LT

%left LT
%left PLUS SUB
%left MUL DIV

%nonassoc IDENT
%nonassoc LPAREN

%start input
%type <KaleidoscopeAST.toplevel> input

%%

input:	statement SEMI	{ $1 }
;

statement:	  e		{ Expression($1) }
			| extern	{ Proto($1) }
			| p		{ Proto($1) }
			| f		{ Func($1) }
;

arguments:	  /* empty */	{ [] }
			| arguments e	{ $1 @ [$2] }
;

arg_defs:	  /* empty */		{ [] }
		| arg_defs IDENT	{ $1 @ [$2] }
;

p_body: IDENT LPAREN arg_defs RPAREN	{ Prototype($1, $3) }
;

/* This rule is for expressions.  It is abreviated to avoid naming conflicts */
e:	  LPAREN e RPAREN				{ $2 }
	| IDENT						{ Variable($1) }
	| NUMBER						{ Number($1) }
	| e PLUS e					{ Binary(Plus, $1, $3) }
	| e SUB e						{ Binary(Sub, $1, $3) }
	| e MUL e						{ Binary(Mul, $1, $3) }
	| e DIV e						{ Binary(Div, $1, $3) }
	| e LT e						{ Binary(Lt, $1, $3) }
	| IDENT LPAREN arguments RPAREN	{ Call($1, $3) }
;

extern:	EXTERN p_body	{ $2 }
;

/* This rule is for prototypes.  It is abreviated to avoid naming conflicts */
p: DEF p_body	{ $2 }
;

/* This rule is for functions.  It is abreviated to avoid naming conflicts */
f: p e	{ Function($1, $2) }
;

%%
