{
	(* This file is the Ocamllex definition of the lexer for the Kaleidoscope language. *)

	(* Standard libraries *)
	open Printf
	
	(* LLVM related libraries *)
	
	(* This project's libraries *)
	open KaleidoscopeParser
	
	exception Error of string
}

let digit = ['0'-'9']
let char = ['a'-'z''A'-'Z']
let whitespace = ['\n''\r'' ''\t']

rule token = parse
	  "def"						{ DEF }
	| "extern"					{ EXTERN }
	| "if"						{ IF }
	| "then"						{ THEN }
	| "else"						{ ELSE }
	| "for"						{ FOR }
	| "in"						{ IN }
	| char+ as i					{ IDENT(i) }
	| digit+ as num
	| "." digit+ as num
	| digit+ "." digit* as num    	{ NUMBER(float_of_string num) }
	| ','						{ COMMA }
	| ';'						{ SEMI }
	| '('						{ LPAREN }
	| ')'						{ RPAREN }
	| '='						{ ASSIGN }
	| '+'						{ PLUS }
	| '-'						{ SUB }
	| '*'						{ MUL }
	| '/'						{ DIV }
	| '<'						{ LT }
	| whitespace					{ token lexbuf }
	| eof						{ raise End_of_file }
	| _							{ token lexbuf }

