(* This file defines the JIT compiler for the Kaleidoscope language using the LLVM *)

(* Standard libraries *)

(* LLVM related libraries *)
open Llvm
open Llvm_executionengine
open Llvm_scalar_opts
open Llvm_target

(* This project's libraries *)

(* Create the JIT and function pass manager *)
let module_provider = ModuleProvider.create KaleidoscopeTranslator.working_module
let execution_engine = ExecutionEngine.create module_provider
let fpm = PassManager.create_function module_provider

let execute f =
	GenericValue.as_float double_type (ExecutionEngine.run_function f [||] execution_engine)

let optimize f =
	let _ = PassManager.run_function f fpm in
		f

(* This initializes the function optimizer. *)
let _ =
	(* Set up the optimizer pipeline.  Start with registering info about how the *)
	(* target lays out data structures. *)
	TargetData.add (ExecutionEngine.target_data execution_engine) fpm;

	(* Do simple "peephole" and bit-twiddling optimizations. *)
	add_instruction_combining fpm;

	(* Reassociate expressions. *)
	add_reassociation fpm;

	(* Eliminate Common SubExpressions. *)
	add_gvn fpm;

	(* Simplify the control flow graph (deleting unreachable blocks, etc). *)
	add_cfg_simplification fpm;

