(* *)

type op =
	  Plus
	| Sub
	| Div
	| Mul
	| Lt
	| Assign

(* expr - Base type for all expression nodes. *)
type expr =
  (* Constructor for numeric literals like "1.0". *)
    Number of float

  (* Constructor for referencing a variable, like "a". *)
  | Variable of string

  (* Constructor for a binary operator. *)
  | Binary of op * expr * expr

  (* Constructor for function calls. *)
  | Call of string * expr list
  
  (* Constructor for variable definitions. *)
  | VarDef of (string * expr) list * expr
  
  (* Constructor for an if expression. *)
  | If of expr * expr * expr
  
  (* Constructor for a for expression. *)
  | For of string * expr * expr * expr * expr

(* proto - This type represents the "prototype" for a function, which        *)
(* captures its name, and its argument names (thus implicitly the number of  *)
(* arguments the function takes).                                            *)
type proto = Prototype of string * string list

(* func - This type represents a function definition itself. *)
type func = Function of proto * expr

type toplevel =
	  Expression of expr
	| Proto of proto
	| Func of func
