{
	(* This file is the Ocamllex definition of the lexer for the Kaleidoscope language. *)

	(* Standard libraries *)
	open Printf
	
	(* LLVM related libraries *)
	
	(* This project's libraries *)
	open KaleidoscopeParser
	
	exception Error of string
	
	let unary_ops:char list ref = ref []
	let binary_ops:char list ref = ref []
	
	let add_uop o =
		unary_ops := o::(!unary_ops)
	
	let add_bop o =
		binary_ops := o::(!binary_ops)
	
	let is_uop o =
		List.exists (fun el -> o == el) !unary_ops
	
	let is_bop o =
		List.exists (fun el -> o == el) !binary_ops
	
	let check_op o =
		if is_uop o then
			UOP(o)
		else
			begin
				if is_bop o then
					BOP(o)
				else
					raise (Error "Undefined operator used.");
			end
}

let digit = ['0'-'9']
let char = ['a'-'z''A'-'Z']
let whitespace = ['\n''\r'' ''\t']
let op = [^ 'a'-'z' 'A'-'Z' '0'-'9' '+' '-' '/' '*' '<' '(' ')' ';' '{' '}']

rule token = parse
	  "def"						{ DEF }
	| "unary" ' ' (op as o)			{ add_uop o; DEFOP(o) }
	| "binary" ' ' (op as o)			{ add_bop o; DEFOP(o) }
	| "extern"					{ EXTERN }
	| "var"						{ VAR }
	| "if"						{ IF }
	| "then"						{ THEN }
	| "else"						{ ELSE }
	| "for"						{ FOR }
	| "in"						{ IN }
	| char+ as i					{ IDENT(i) }
	| digit+ as num
	| "." digit+ as num
	| digit+ "." digit* as num		{ NUMBER(float_of_string num) }
	| ','						{ COMMA }
	| ';'						{ SEMI }
	| '('						{ LPAREN }
	| ')'						{ RPAREN }
	| '='						{ ASSIGN }
	| '+'						{ PLUS }
	| '-'						{ SUB }
	| '*'						{ MUL }
	| '/'						{ DIV }
	| '<'						{ LT }
	| whitespace					{ token lexbuf }
	| op as o						{ check_op o }
	| eof						{ raise End_of_file }
	| _							{ token lexbuf }

