%{
	(* This file is the Ocamlyacc definition of the parser for the Kaleidoscope language. *)
	
	(* Standard libraries *)
	open Printf
	
	(* LLVM related libraries *)
	
	(* This project's libraries *)
	open KaleidoscopeAST
%}

%token <string> IDENT
%token <float> NUMBER
%token <char> UOP BOP
%token <char> DEFOP
%token DEF EXTERN VAR
%token IF THEN ELSE FOR IN
%token SEMI LPAREN RPAREN COMMA
%token ASSIGN
%token PLUS SUB MUL DIV
%token LT

%left IN ELSE
%left LT BOP
%right ASSIGN
%left PLUS SUB
%left MUL DIV
%left UOP

%nonassoc IDENT
%nonassoc LPAREN

%start input
%type <KaleidoscopeAST.toplevel> input

%%

input:	statement SEMI	{ $1 }
;

statement:	  e		{ Expression($1) }
			| extern	{ Proto($1) }
			| p		{ Proto($1) }
			| f		{ Func($1) }
;

arguments:	  /* empty */	{ [] }
			| arguments e	{ $1 @ [$2] }
;

arg_defs:	  /* empty */		{ [] }
		| arg_defs IDENT	{ $1 @ [$2] }
;

increment_value:	  /* empty */	{ Number(1.0) }
				| COMMA e		{ $2 }
;

for_loop:	FOR IDENT ASSIGN e COMMA e increment_value IN e	{ For($2, $4, $6, $7, $9)  }
;

var_def_listA:	  var_def_listB IDENT			{ $1 @ [($2, Number(0.0))] }
			| var_def_listB IDENT ASSIGN e	{ $1 @ [($2, $4)]}
;

var_def_listB:	  /* empty */						{ [] }
			| var_def_listB IDENT COMMA			{ $1 @ [($2, Number(0.0))] }
			| var_def_listB IDENT ASSIGN e COMMA	{ $1 @ [($2, $4)]}
;

var_def: VAR var_def_listA IN e	{ VarDef($2, $4) }
;

p_body: IDENT LPAREN arg_defs RPAREN	{ Prototype($1, $3) }
;

/* This rule is for expressions.  It is abreviated to avoid naming conflicts */
e:	  LPAREN e RPAREN				{ $2 }
	| IDENT						{ Variable($1) }
	| NUMBER						{ Number($1) }
	| for_loop					{ $1 }
	| IF e THEN e ELSE e			{ If($2, $4, $6) }
	| var_def						{ $1 }
	| IDENT ASSIGN e				{ Binary(Assign, Variable($1), $3) }
	| e BOP e						{ Call("op" ^ (String.make 1 $2), [$1;$3]) }
	| e PLUS e					{ Binary(Plus, $1, $3) }
	| e SUB e						{ Binary(Sub, $1, $3) }
	| e MUL e						{ Binary(Mul, $1, $3) }
	| e DIV e						{ Binary(Div, $1, $3) }
	| e LT e						{ Binary(Lt, $1, $3) }
	| IDENT LPAREN arguments RPAREN	{ Call($1, $3) }
	| UOP e						{ Call("op" ^ (String.make 1 $1), [$2]) }
;

extern:	EXTERN p_body	{ $2 }
;

/* This rule is for prototypes.  It is abreviated to avoid naming conflicts */
p: DEF p_body	{ $2 }
;

/* This rule is for functions.  It is abreviated to avoid naming conflicts */
f:	  p e						{ Function($1, $2) } 
	| DEFOP LPAREN arg_defs RPAREN e	{ Function(Prototype("op" ^ (String.make 1 $1), $3), $5) }
;

%%
