open Printf

open KaleidoscopeParser

let print_token token =
		match token with
			  IDENT(s) -> Printf.printf "Ident(%s) " s
			| NUMBER(x) -> Printf.printf "Number(%f) " x
			| BOP(c) -> Printf.printf "BOp(%c) " c
			| UOP(c) -> Printf.printf "UOp(%c) " c
			| DEF -> print_string "Def "
			| DEFOP(c) -> Printf.printf "Defop(%c) " c;
			| IN -> print_string "in "
			| FOR -> print_string "For "
			| IF -> print_string "If "
			| ELSE -> print_string "else "
			| THEN -> print_string "then "
			| EXTERN -> print_string "Extern "
			| LPAREN -> print_string "("
			| RPAREN -> print_string ") "
			| COMMA -> print_string ","
			| PLUS -> print_string "+ "
			| SUB -> print_string "- "
			| MUL -> print_string "* "
			| DIV -> print_string "/ "
			| LT -> print_string "< "
			| ASSIGN -> print_string "= "
			| SEMI -> print_string "\b;\nready>"; flush stdout

let main () =
	try
		let lexbuf = Lexing.from_channel stdin in
			print_string "ready> "; flush stdout;
			while true do
				print_token (KaleidoscopeLexer.token lexbuf);
			done
	with End_of_file -> exit 0

let _ = Printexc.print main ()
